;;; -*- asm -*-
;;;
;;; Copyright (c) 2004  Robert James Kaes <rjk@wormbytes.ca>
;;;   All rights reserved.
;;;
;;;
;;; This program is free software; you can redistribute it and/or modify it
;;;  under the terms of the GNU General Public License as published by the
;;; Free Software Foundation; either version 2, or (at your option) any
;;; later version.
;;;
;;; This program is distributed in the hope that it will be useful, but
;;; WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;;;  General Public License for more details.
;;; 

        include "processor.inc"

.display_rpm_data       udata
rpm             res     1
leds_lit        res     1
temp            res     1

.display_rpm_code       code

display_rpm:
        global  display_rpm

        banksel rpm
        movwf   rpm             ; store of a copy of the rpm
        movwf   temp

        ;; calculate the number of LEDs to show
        clrf    leds_lit
count_leds_loop:       
        movlw   5
        subwf   temp, F
        bn      led_max_allowed
        incf    leds_lit, F
        bra     count_leds_loop

led_max_allowed:        
        movlw   16              ; can't have more than 16 lights
        cpfslt  leds_lit        ; skip if leds_lit < 16
        movwf   leds_lit

light_led_loop:
        set_pclat lightpattern
        
        movfw   leds_lit
        call    lightpattern    ; get the light pattern for PORTC
        movwf   PORTC           ; W contains the bit pattern

        ;; Display the RPM on the seven segment.  Anything over 99
        ;; is classified as an OverLoad.
        banksel rpm
        movlw   99
        cpfsgt  rpm             ; skip if rpm > 99
        bra     show_normal

        ;; it's an overload condition
        call    show_overload
        bra     check_leds_lit

show_normal:
        movfw   rpm
        call    show_seven_seg

check_leds_lit: 
        ;; check if the number of lights is zero
        banksel leds_lit
        movfw   leds_lit
        skpnz
        return

        decfsz  leds_lit, F
        bra     light_led_loop
        
        return


.sevenseg_data   udata
num_delay       res     1
num             res     1
div             res     1               ; store the results of the divide
display_hold    res     1

.sevenseg_code  code

show_seven_seg:
        banksel num
        movwf   num             ; save the original number
        movwf   temp

        ;; divide the num by 10
        clrf    div
divide_loop:
        movlw   10
        subwf   temp, F
        bn      divide_finish   ; finished when number goes negative
        
        incf    div, F
        bra     divide_loop

divide_finish:
        ;; div now contains the tens position value
        bsf     PORTA, RA3
        bcf     PORTA, RA2
        movfw   div
        call    display_number

        ;; now calculate the ones position number
        movlw   10
        mulwf   div
        movfw   PRODL
        subwf   num, W

        bcf     PORTA, RA3
        bsf     PORTA, RA2
        call    display_number
        return


;;; --------------------------------------------------------------------
;;;
;;; Show the value "OL" on the seven segment, since this is an overload
;;; condition
;;;
show_overload:
        ;; show the "0"
        bsf     PORTA, RA3
        bcf     PORTA, RA2
        clrw
        call    display_number

        ;; now show the L
        bcf     PORTA, RA3
        bsf     PORTA, RA2
        movlw   10              ; 10th position is "L"
        call    display_number
        return
        

display_number:
        movwf   display_hold

        set_pclat numberpattern

        clrf    PORTB
        movfw   display_hold
        call    numberpattern
        movwf   PORTB

        movlw   40h
        movwf   num_delay
        decfsz  num_delay, F
        bra     $ - 2
        return

.patterns_code  code    500h
        
lightpattern:
        rlncf   WREG, W
        addwf   PCL, F
        retlw   b'11111111'     ; no lights
        retlw   b'00010111'     ; 0
        retlw   b'00100111'     ; 1
        retlw   b'01000111'     ; 2
        retlw   b'10000111'     ; 3
        retlw   b'00011011'     ; 4
        retlw   b'00101011'     ; 5
        retlw   b'01001011'     ; 6
        retlw   b'10001011'     ; 7
        retlw   b'00011101'     ; 8
        retlw   b'00101101'     ; 9
        retlw   b'01001101'     ; 10
        retlw   b'10001101'     ; 11
        retlw   b'00011110'     ; 12
        retlw   b'00101110'     ; 13
        retlw   b'01001110'     ; 14
        retlw   b'10001110'     ; 15

numberpattern:
        rlncf   WREG, W
        addwf   PCL, F
        retlw   b'01110111'     ; 0
        retlw   b'01000001'     ; 1
        retlw   b'01101110'     ; 2
        retlw   b'01101101'     ; 3
        retlw   b'01011001'     ; 4
        retlw   b'00111101'     ; 5
        retlw   b'00111111'     ; 6
        retlw   b'01100001'     ; 7
        retlw   b'01111111'     ; 8
        retlw   b'01111001'     ; 9
        retlw   b'00010110'     ; "L"
        
        end
        