;;; -*- asm -*-
;;;
;;; Copyright (c) 2004  Robert James Kaes <rjk@wormbytes.ca>
;;;   All rights reserved.
;;;
;;;
;;; This program is free software; you can redistribute it and/or modify it
;;;  under the terms of the GNU General Public License as published by the
;;; Free Software Foundation; either version 2, or (at your option) any
;;; later version.
;;;
;;; This program is distributed in the hope that it will be useful, but
;;; WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;;;  General Public License for more details.
;;; 

#ifndef MACRO_H
#define MACRO_H


;;; ------------------------------------------------------------------
;;; Convience routines

;;; Move a register value into W
movfw	macro	reg
	movf	(reg), W
	endm

;;; Set the upper two bytes of the PCLAT* registers based on the
;;; location supplied.
set_pclat       macro   loc
        movlw   UPPER(loc)
        movwf   PCLATU
        movlw   HIGH(loc)
        movwf   PCLATH
        endm
	

;;; ------------------------------------------------------------------
;;; Block memory routines

;;; Set a section of memory to a predefined constant value
memset	macro	mem, val, count
	local index = 0
	while index < count
	  if val == 0
	clrf	(mem) + index
	  else
	    if val == 0ffh
	setf	(mem) + index
	    else
	movlw	val
	movwf	(mem) + index
	    endif
	  endif
index = index + 1
	endw
	endm

;;; Clear a section of memory starting at MEM and going for COUNT bytes
memclr	macro	mem, count
	memset	(mem), 0, count
	endm

;;; Increment the memory location by one
meminc	macro	mem, count
	local index = 0
	while index < count
	incf	(mem) + index, F
	
	if (index < count - 1)
	skpnz
	endif
	
index = index + 1
	endw
	endm

;;; Decrement the memory location by one
memdec	macro	mem, count
	;; substract one and then propegate the borrow
	movlw	1
	subwf	(mem), F
	clrw

	local index = 1
	while index < count
	subwfb	(mem) + index, F
index = index + 1
	endw
	endm

;;; Compare a range of memory to a literal value.  The Z status flag
;;; is set.  If the memory equals the literal, Z is set; otherwise, Z
;;; is cleared.
memcmpl	macro	mem, lit, len
	local	_exit

	banksel	(mem)	
	local index = 0
	while index < len
	movlw	lit
	subwf	(mem) + index, W
	bnz	_exit
index = index + 1
	endw
_exit:	
	endm


;;; Add two blocks of memory together
;;; to = to + from
memadd	macro	to, from, len
	local index = 0
	while index < len
	movfw	(from) + index
	 if index < 1
	addwf	(to) + index, F
	 else
	addwfc	(to) + index, F
	 endif
index = index + 1
	endw
	endm

#endif
	